@@ -1,3 +1,17 @@
+# Copyright (c) 2010 Daniel Would
+# Licensed under the Apache License, Version 2.0 (the "License");
+#   you may not use this file except in compliance with the License.
+#   You may obtain a copy of the License at
+#
+#       http://www.apache.org/licenses/LICENSE-2.0
+#
+#  Unless required by applicable law or agreed to in writing, software
+#  distributed under the License is distributed on an "AS IS" BASIS,
+#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#  See the License for the specific language governing permissions and
+#  limitations under the License.
+#
+#
 #account class representing all the objects
 #for access to an account and associated runtime tweets/etc
 
@@ -12,6 +26,10 @@ import urllib
 import simplejson
 import bitly
 import accountData
+import os.path
+import gobject
+import sys, os, time
+from stat import *
 
 class account():
 
@@ -22,31 +40,40 @@ class account():
     ACTIVE = 1
     INACTIVE = 2
     status = ""
+    avatar_size = 60
 
-    def __init__(self, osso_c, accData):
+    def __init__(self, osso_c, accData, controller):
+        self.controller = controller
         #these store all the data associated with this accounts tweets
-        # the fields are : Name,nameColour,Tweet,TweetColour,id, type, timestamp, replyTo, source
-        self.tweetstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
+        # the fields are : Name,sender_id,Tweet,TweetColour,id, type, timestamp, replyTo, source
+        self.tweetstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
         #then we want the same again to store dm's, mentions & pubilc timeline separately
-        self.dmstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.mentionstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.publicstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.trendstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.friendsstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.searchstore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
-        self.userhistorystore = gtk.ListStore(str, str, str, str, str, str, str, str, str)
+        self.dmstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.mentionstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.publicstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.trendstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.friendsstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.searchstore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        self.userhistorystore = gtk.ListStore(str, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str)
+        #store of avatar, filename, access count
+        self.avatars = gtk.ListStore(gtk.gdk.Pixbuf, str, int, str)
         #base url can be set to any twitter compatible site, so we can support mutliple account types
         self.accountdata = accData
         self.osso_c = osso_c
         self.api = None
-        if accData.password:
-            self.api = twitter.Api(username=accData.username, password=accData.password)
-            self.api.SetBaseUrl(accData.baseUrl)
-            self.api.SetBaseSearchUrl(accData.searchUrl)
-        if (accData.access_token != None):
-            self.api = oauthtwitter.OAuthApi(self.CONSUMER_KEY, self.CONSUMER_SECRET, accData.access_token)
-
+        self.connect()
 
+    def connect(self):
+        if self.accountdata.password:
+            self.api = twitter.Api(username=self.accountdata.username, password=self.accountdata.password)
+            self.api.SetBaseUrl(self.accountdata.baseUrl)
+            self.api.SetBaseSearchUrl(self.accountdata.searchUrl)
+        if (self.accountdata.access_token != None):
+            self.api = oauthtwitter.OAuthApi(self.CONSUMER_KEY, self.CONSUMER_SECRET, self.accountdata.access_token)
+        if (self.api == None):
+            return False
+        else:
+            return True
 
     def getAccountData(self):
         return self.accountdata
@@ -75,6 +102,8 @@ class account():
         return self.accountdata.searchUrl
     def getAccessToken(self):
         return self.accountdata.access_token
+    def setAccessToken(self, access_token):
+        self.accountdata.access_token = access_token
 
     def setStatus(self, status):
         self.accountdata.status = status
@@ -82,6 +111,10 @@ class account():
     def setBitlyCreds(self, username, apikey):
         self.accountdata.bitlyusername = username
         self.accountdata.bitlyapikey = apikey
+    def getBitlyUid(self):
+        return self.accountdata.bitlyusername
+    def getBitlyapikey(self):
+        return self.accountdata.bitlyapikey
 
     def setServiceCreds(self, username, password=None, access_token=None):
         self.accountdata.username = username
@@ -119,8 +152,8 @@ class account():
 
     def getTweets(self, auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting tweets with " + self.accountdata.username
         print "base url = " + self.accountdata.baseUrl
         print "base url of api object = " + self.api.GetBaseUrl()
@@ -169,8 +202,10 @@ class account():
                  #strip the source app name from the url
                  source = self.getSourceAppname(x.source)
                  text = self.escapeText(x.text)
-                 text = self.expandBitlyUrls(text)
-                 self.tweetstore.append([ "@" + x.user.screen_name, "", "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source])
+                 text = self.controller.expandBitlyUrls(text)
+                 pic = self.set_pic_for_id(str(x.user.GetId()), x.user.GetScreenName(), x.user.GetProfileImageUrl())
+                 formatted_tweet = self.format_tweet(x.user.screen_name, text, x.created_at, source, reply_to)
+                 self.tweetstore.append([ "@" + x.user.screen_name, str(x.user.GetId()), "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source, pic, formatted_tweet])
                  receive_count = receive_count + 1
 
             if (receive_count > 0):
@@ -200,8 +235,8 @@ class account():
 
     def getMentions(self, auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting mentions"
         receive_count = 0
         try:
@@ -248,8 +283,10 @@ class account():
                  #strip the source app name from the url
                  source = self.getSourceAppname(x.source)
                  text = self.escapeText(x.text)
-                 text = self.expandBitlyUrls(text)
-                 self.mentionstore.append([ "@" + x.user.screen_name, "", "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source])
+                 text = self.controller.expandBitlyUrls(text)
+                 pic = self.set_pic_for_id(str(x.user.GetId()), x.user.GetScreenName(), x.user.GetProfileImageUrl())
+                 formatted_tweet = self.format_tweet(x.user.screen_name, text, x.created_at, source, reply_to)
+                 self.mentionstore.append([ "@" + x.user.screen_name, str(x.user.GetId()), "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source, pic, formatted_tweet])
                  receive_count = receive_count + 1
 
             if (receive_count > 0):
@@ -277,8 +314,8 @@ class account():
 
     def getDMs(self, auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting dms"
         receive_count = 0
         try:
@@ -317,8 +354,10 @@ class account():
                         self.accountdata.oldest_dm_id = x.id
                  user = x.GetSenderScreenName()
                  text = self.escapeText(x.text)
-                 text = self.expandBitlyUrls(text)
-                 self.dmstore.append([ "@" + user, "", "@" + user + " : " + text, "", x.id, "Tweet", x.created_at, "", ""])
+                 text = self.controller.expandBitlyUrls(text)
+                 pic = self.set_pic_for_id(str(x.GetSenderId()), x.GetSenderScreenName(), "")
+                 formatted_tweet = self.format_tweet(user, text, x.created_at, None, None)
+                 self.dmstore.append([ "@" + user, x.GetSenderId(), "@" + user + " : " + text, "", x.id, "Tweet", x.created_at, "", "", pic, formatted_tweet])
                  receive_count = receive_count + 1
 
             if (receive_count > 0):
@@ -346,8 +385,8 @@ class account():
 
     def getFriends(self, auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting Friends"
         receive_count = 0
         self.friendsstore.clear()
@@ -367,7 +406,10 @@ class account():
                         tweettime = ""
                         status = ""
                     status = self.escapeText(status)
-                    self.friendsstore.append([ "@" + x.screen_name, "", "@" + x.screen_name + " : " + status, "", x.id, "friend", tweettime, "", source])
+                    source = self.getSourceAppname(source)
+                    pic = self.set_pic_for_id(str(x.id), x.GetScreenName(), x.GetProfileImageUrl())
+                    formatted_tweet = self.format_tweet(x.screen_name, status, tweettime, source, None)
+                    self.friendsstore.append([ "@" + x.screen_name, str(x.id), "@" + x.screen_name + " : " + status, "", x.id, "friend", tweettime, "", source, pic, formatted_tweet])
                     receive_count = receive_count + 1
 
                 if (receive_count > 0):
@@ -395,8 +437,8 @@ class account():
 
     def getPublic(self, auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting public"
         receive_count = 0
         try:
@@ -443,8 +485,10 @@ class account():
                  #strip the source app name from the url
                  source = self.getSourceAppname(x.source)
                  text = self.escapeText(x.text)
-                 text = self.expandBitlyUrls(text)
-                 self.publicstore.append([ "@" + x.user.screen_name, "", "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source])
+                 text = self.controller.expandBitlyUrls(text)
+                 pic = self.set_pic_for_id(str(x.user.GetId()), x.user.GetScreenName(), x.user.GetProfileImageUrl())
+                 formatted_tweet = self.format_tweet(x.user.screen_name, text, x.created_at, source, reply_to)
+                 self.publicstore.append([ "@" + x.user.screen_name, x.user.GetId(), "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source, pic, formatted_tweet])
                  receive_count = receive_count + 1
 
             if (receive_count > 0):
@@ -472,8 +516,8 @@ class account():
 
     def getUserHistory(self, friend="", auto=0, older=False, get_count=20, * args):
         if (self.api == None):
-            print "account not configured"
-            return
+            if (self.connect() != True):
+                return
         print "getting User history for " + friend
         self.userhistorystore.clear()
         receive_count = 0
@@ -495,8 +539,10 @@ class account():
                  #strip the source app name from the url
                  source = self.getSourceAppname(x.source)
                  text = self.escapeText(x.text)
-                 text = self.expandBitlyUrls(text)
-                 self.userhistorystore.append([ "@" + x.user.screen_name, "", "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source])
+                 text = self.controller.expandBitlyUrls(text)
+                 pic = self.set_pic_for_id(str(x.user.GetId()), x.user.GetScreenName(), x.user.GetProfileImageUrl())
+                 formatted_tweet = self.format_tweet(x.user.screen_name, text, x.created_at, source, reply_to)
+                 self.userhistorystore.append([ "@" + x.user.screen_name, x.user.GetId(), "@" + x.user.screen_name + " : " + text, "", x.id, "Tweet", x.created_at, reply_to, source, pic, formatted_tweet])
                  receive_count = receive_count + 1
 
             if (receive_count > 0):
@@ -526,7 +572,9 @@ class account():
         print "performing search"
         self.searchstore.clear()
         receive_count = 0
-
+        #when manually triggered use the current text in the entry field
+        if (auto == 0):
+            searchTerms = self.controller.ui.getEntryText()
         if (searchTerms == ""):
             print "nothing to search"
             return
@@ -556,10 +604,13 @@ class account():
                     reply_to = ""
                     reply_to = reply_to.replace("&", "&amp;")
                     text = self.escapeText(x['text'])
-                    text = self.expandBitlyUrls(text)
+                    text = self.controller.expandBitlyUrls(text)
                     #strip the source app name from the url
                     source = self.getSourceAppname(x['source'])
-                    self.searchstore.append([ "@" + x['from_user'], "", "@" + x['from_user'] + " : " + text, "", x['id'], "Search", x['created_at'], reply_to, source])
+                    print "search result from userid: " + str(x['from_user_id'])
+                    pic = self.get_pic_for_name(str(x['from_user']))
+                    formatted_tweet = self.format_tweet(x['from_user'], text, x['created_at'], source, reply_to)
+                    self.searchstore.append([ "@" + x['from_user'], str(x['from_user_id']), "@" + x['from_user'] + " : " + text, "", x['id'], "Search", x['created_at'], reply_to, source, pic, formatted_tweet])
                     receive_count = receive_count + 1
                     note = osso.SystemNote(self.osso_c)
                     result = note.system_note_infoprint("Search results Received for : " + term)
@@ -595,7 +646,7 @@ class account():
             #and the values of top level values text/id/created_at
             trends = data['trends']
             for x in trends:
-                self.trendstore.append([x['name'], "", x['name'] + " :" + x['url'], "", "", "dm", "", "", ""])
+                self.trendstore.append([x['name'], "", x['name'] + " :" + x['url'], "", "", "dm", "", "", "", None, x['name'] + " :" + x['url']])
 
             note = osso.SystemNote(self.osso_c)
 
@@ -738,30 +789,146 @@ class account():
         text = text.replace("&", "&amp;")
         return text
 
-    def expandBitlyUrls(self, data):
+    def get_pic_for_name (self, name):
+        item = self.avatars.get_iter_first()
+        while (item != None):
+            print "comparing " + self.avatars.get_value(item, 3) + " to " + name
+            if (self.avatars.get_value(item, 3) == name):
+                return self.avatars.get_value(item, 0)
+            item = self.avatars.iter_next(item)
+        return gtk.gdk.pixbuf_new_from_file("/opt/witter/icons/tweet.png")
 
-        if (self.accountdata.bitlyusername != ""):
-                words = data.split(" ")
-                for word in words:
-                    if re.search("http://bit.ly", word):
-                        #appears to have a bitly URL so lets lengthen it
-                        try:
-                            bitlyapi = bitly.Api(login=self.accountdata.bitlyusername, apikey=self.accountdata.bitlyapikey)
+    def set_pic_for_id(self, id, name, url):
+        return self.set_pix(id + ".jpg", url, id, name)
 
-                            longurl = bitlyapi.expand(word)
-                            #having gotten the long url
-                            #strip out the domain name and append it to the tweet
-                            #after the bit.ly url in brackets
-                            enddomain = int(longurl.find("/", 7, len(longurl)))
-                            print enddomain
-                            longurl = longurl[7:enddomain]
+    def set_pix(self, filename, url, id, name):
+        #print "looking for image /home/user/.witterPics/" + filename
+        #print "profile pic url " + url
+        item = self.avatars.get_iter_first ()
+        #first find out if we already loaded this avatar
+        while (item != None):
+            pic = (self.avatars.get_value (item, 1))
 
-                            #switch the long url in to the data
+            if (pic == filename):
+                #if we access a user more than 10 times, reload their avatar
+                if (self.avatars.get_value(item, 2) > 10):
+                    print "refreshing avatar for " + id
+                    pixbuf, success = self.retrieve_avatar(url, filename)
+                    if (success):
+                        self.avatars.set_value(item, 0, pixbuf)
+                        self.avatars.set_value(item, 2, 0)
+                        self.update_all_avatars(id, pixbuf)
+                    return pixbuf
+                else:
+                    print "re-using file already loaded"
+                    self.avatars.set_value(item, 2, (self.avatars.get_value(item, 2) + 1))
+                    return self.avatars.get_value(item, 0)
+            item = self.avatars.iter_next(item)
+        #if we make it here we didn't have it loaded already
+        if (os.path.isfile("/home/user/.witterPics/" + filename)):
+            pixbuf = gtk.gdk.pixbuf_new_from_file("/home/user/.witterPics/" + filename)
+            pixbuf = pixbuf.scale_simple(self.avatar_size, self.avatar_size, gtk.gdk.INTERP_BILINEAR)
+            self.avatars.append([pixbuf, filename, 1, name])
+            print "loaded pixbuf from file"
+            return pixbuf
+        else:
+            try:
+                pixbuf, success = self.retrieve_avatar(url, filename)
+                #urllib.urlretrieve(url, " / home / user / .witterPics / " + filename)
+                #print "profile pic retrieved"
+                if (success == True):
+                    self.avatars.append([pixbuf, filename, 1, name])
+                    self.update_all_avatars(id, pixbuf)
+                else:
+                    print "assiging temp pixbuf"
+            except IOError, e:
+                print "io error"
+                print e
+                print "failed to find avatar for user: " + filename
+                pixbuf = gtk.gdk.pixbuf_new_from_file("/opt/witter/icons/tweet.png")
+        return pixbuf
+
+    def retrieve_avatar(self, url, filename):
+        try:
+            if (os.path.isdir("/home/user/.witterPics") != True):
+                os.mkdir("/home/user/.witterPics")
+            if (os.path.isdir("/home/user/.witterPics/" + self.accountdata.servicename) != True):
+                os.mkdir("/home/user/.witterPics/" + self.accountdata.servicename)
+            webFile = urllib.urlopen(url)
+            print "fetched image"
+            localFile = open("/home/user/.witterPics/" + self.accountdata.servicename + "/" + filename, 'w')
+            localFile.write(webFile.read())
+            webFile.close()
+            localFile.close()
+            pixbuf = gtk.gdk.pixbuf_new_from_file("/home/user/.witterPics/" + self.accountdata.servicename + "/" + filename)
+            pixbuf = pixbuf.scale_simple(self.avatar_size, self.avatar_size, gtk.gdk.INTERP_BILINEAR)
+        except IOError, e:
+            print e
+            print "failed to retreive picture for " + url + " : " + filename
+            pixbuf = gtk.gdk.pixbuf_new_from_file("/opt/witter/icons/tweet.png")
+            return pixbuf, False
+
+        return pixbuf, True
+
+    def format_tweet(self, user, tweet, time, source, reply_to):
+        tweet = gobject.markup_escape_text(tweet)
+        if (source != None):
+            source = gobject.markup_escape_text(source)
+        line = " <span weight = 'bold' foreground=\"#6bd3ff\">" + user + " : </span> "
+
+        words = tweet.split(" ")
+        for word in words:
+            if (word.startswith("@")):
+                word = " <span foreground = \"#0075b5\">" + word + "</span>"
+            if (word.startswith("http:")):
+                word = "<span foreground=\"#0075b5\">" + word + "</span>"
+            if (word.startswith("witter") | word.startswith("Witter") | word.startswith("#witter") | word.startswith("#Witter")):
+                word = "<span foreground=\"yellow\">" + word + "</span>"
+            line = line + " " + word
+
+        if (reply_to != None):
+            if (reply_to != ""):
+                line = line + "\n<span size='small'> " + reply_to + "</span>"
+        if (source != None):
+            if (source.startswith("Witter")):
+                source = "<span foreground=\"yellow\">" + source + "</span>"
+            line = line + "\n<span weight='light' size='x-small'>" + time + " from " + source + "</span>"
+        else:
+            line = line + "\n<span weight='light' size='x-small'>" + time + "</span>"
+        return line
+
+    def get_age(self, filename):
+        now = time.time()
+        try:
+            st = os.stat(filename)
+
+        except os.error, msg:
+            sys.stderr.write("can't stat %r: %r\n" % (filename, msg))
+            status = 1
+            st = ()
+            return 10
+        if st:
+            anytime = st[ST_MTIME]
+            fileage = time.strftime('%a %d %b/%Y %H:%M', time.localtime(anytime))
+
+            print "file ages is " + str(fileage)
+        return fileage
 
 
-                            data = data.replace(word, word + " (" + longurl + ")")
-                        except bitly.BitlyError:
-                            print "bitly gave error response"
-                        except KeyError:
-                            print "bitly gave key error response"
-        return data
+    def update_all_avatars(self, id, pixbuf):
+        print "updating all stored images for " + id
+        self.update_avatars(id, pixbuf, self.tweetstore)
+        self.update_avatars(id, pixbuf, self.mentionstore)
+        self.update_avatars(id, pixbuf, self.dmstore)
+        self.update_avatars(id, pixbuf, self.publicstore)
+        self.update_avatars(id, pixbuf, self.friendsstore)
+        self.update_avatars(id, pixbuf, self.searchstore)
+    def update_avatars(self, id, pixbuf, store):
+        #updates all instances of tweets from the specified id with the pixbuf
+        item = store.get_iter_first()
+        while item != None:
+            if (store.get_value(item, 1) == id):
+                print ("Found pixbuf to update")
+                store.set_value(item, 9, pixbuf)
+            item = store.iter_next(item)
+
