@@ -309,52 +309,78 @@ class WitterUI():
 
         # create the TreeView using treestore this is the object which displays the
         # info stored in the liststore
-        self.treeview = gtk.TreeView(self.liststore)
+        #self.treeview = gtk.TreeView(self.liststore)
+        self.treeview = hildon.GtkTreeView(gtk.HILDON_UI_MODE_NORMAL)
+
         #self.treeview = hildon.hildon_gtk_tree_view_new(self.liststore)
         self.treeview.set_model(self.liststore)
         # create the TreeViewColumn to display the data, I decided on two colums
         # one for name and the other for the tweet
         #self.tvcname = gtk.TreeViewColumn('Name')
-        self.cell = witter.witter_cell_renderer.witterCellRender()
+        #self.cell = witter.witter_cell_renderer.witterCellRender()
         #cell_text = gtk.CellRendererText()
-        #self.cell = gtk.CellRendererText()
+        self.cell = gtk.CellRendererText()
 
         color = self.bg_top_color
 
         color2 = self.bg_bottom_color
 
 
-        self.cell.set_property('backgroundt_r', color.red)
-        self.cell.set_property('backgroundt_g', color.green)
-        self.cell.set_property('backgroundt_b', color.blue)
-        self.cell.set_property('backgroundt_p', color.pixel)
-        self.cell.set_property('backgroundb_r', color2.red)
-        self.cell.set_property('backgroundb_g', color2.green)
-        self.cell.set_property('backgroundb_b', color2.blue)
-        self.cell.set_property('backgroundb_p', color2.pixel)
-        self.cell.set_property('font_size', self.font_size)
+        #self.cell.set_property('backgroundt_r', color.red)
+        #self.cell.set_property('backgroundt_g', color.green)
+        #self.cell.set_property('backgroundt_b', color.blue)
+        #self.cell.set_property('backgroundt_p', color.pixel)
+        #self.cell.set_property('backgroundb_r', color2.red)
+        #self.cell.set_property('backgroundb_g', color2.green)
+        #self.cell.set_property('backgroundb_b', color2.blue)
+        #self.cell.set_property('backgroundb_p', color2.pixel)
+        #self.cell.set_property('font_size', self.font_size)
 
-        self.tvctweet = gtk.TreeViewColumn('Pango Markup', self.cell, markup=2)
+        self.tvctweet = gtk.TreeViewColumn('Pango Markup', self.cell, markup=10)
+
+        #Column 0 for the treeview
+        renderer = gtk.CellRendererPixbuf()
+        renderer.set_property('yalign', 0)
+        #renderer.set_property('ypad', 10)
+        #renderer.set_property('cell-background-gdk', gtk.gdk.color_parse("#6bd3ff"))
+
+        column = gtk.TreeViewColumn()
+        column.pack_start(renderer, True)
+        column.add_attribute(renderer, "pixbuf", 9)
+        self.treeview.append_column(column)
+
+        #Column 1 for the treeview
+        renderer = gtk.CellRendererText()
+
+        self.tvctweet.set_property("expand", True)
+        self.cell.set_property('wrap-mode', "word")
+        self.cell.set_property('wrap-width', 730)
+        self.cell.set_property('size-points', self.controller.font_size)
 
 
+        self.treeview.append_column(self.tvctweet)
+        self.treeview.set_rules_hint(True)
+        self.treeview.set_property('enable-grid-lines', True)
         #self.tvctweet = gtk.TreeViewColumn('Tweet')
         # add the two tree view columns to the treeview
         #self.treeview.append_column(self.tvcname)
-        self.treeview.append_column(self.tvctweet)
+
+
 
         # next we add a mapping to the tweet column, again the third field
         # in our list store is the tweet text
-        self.tvctweet.add_attribute(self.cell, 'text', 2)
-        self.tvctweet.add_attribute(self.cell, 'timestamp', 6)
-        self.tvctweet.add_attribute(self.cell, 'replyto', 7)
-        self.tvctweet.add_attribute(self.cell, 'source', 8)
+        self.tvctweet.add_attribute(self.cell, 'text', 10)
+        #self.tvctweet.add_attribute(self.cell, 'timestamp', 6)
+        #self.tvctweet.add_attribute(self.cell, 'replyto', 7)
+        #self.tvctweet.add_attribute(self.cell, 'source', 8)
 
         # make it searchable (I found this in an example and thought I might use it
         # but currently I make no use of this setting
         self.treeview.set_search_column(2)
         self.treeview.set_rules_hint(True)
 
-        self.treeview.set_property('enable-grid-lines', True)
+        #self.treeview.set_property('enable-grid-lines', True)
+
         # Allow sorting on the column. This is cool because no matter what order
         # we load tweets in, we always get a view which is sorted by the tweet id which
         # always increments, so we get them in order
@@ -465,11 +491,11 @@ class WitterUI():
         curView = self.getCurrentView()
         if event.keyval == gtk.keysyms.F7:
             print "zoom in"
-            if (self.font_size < 30):
+            if (self.controller.font_size < 30):
                 self.treeview.hide()
 
-                self.font_size = self.font_size + 1
-                self.cell.set_property('font_size', self.font_size)
+                self.controller.font_size = self.controller.font_size + 1
+                self.cell.set_property('size-points', self.controller.font_size)
                 if (curView == self.TIMELINE_VIEW):
                     self.treeview.set_model(None)
                     self.treeview.set_model(self.controller.activeAccount.getTimeline())
@@ -498,11 +524,11 @@ class WitterUI():
 
         elif event.keyval == gtk.keysyms.F8:
             print "zoom out"
-            if (self.font_size > 10):
+            if (self.controller.font_size > 10):
                 self.treeview.hide()
 
-                self.font_size = self.font_size - 1
-                self.cell.set_property('font_size', self.font_size)
+                self.controller.font_size = self.controller.font_size - 1
+                self.cell.set_property('size-points', self.controller.font_size)
                 if (curView == self.TIMELINE_VIEW):
                     self.treeview.set_model(None)
                     self.treeview.set_model(self.controller.activeAccount.getTimeline())
@@ -726,6 +752,146 @@ class WitterUI():
         self.builder.get_object("hbuttonbox-act4").show_all()
         self.scrollToItem(selectedItem)
 
+    def build_stacked_menu(self, widget, *args):
+        #first hide the other controls
+        print "building stacked menu"
+
+        hbox1 = gtk.HButtonBox()
+        #empty the dynamic buttons
+        buttons = self.builder.get_object("hbuttonbox-act3").get_children()
+        for button in buttons:
+            self.builder.get_object("hbuttonbox-act3").remove(button)
+            button.destroy()
+        buttons2 = self.builder.get_object("hbuttonbox-act4").get_children()
+        for button in buttons2:
+            self.builder.get_object("hbuttonbox-act4").remove(button)
+            button.destroy()
+        #we need to track how many buttons we create
+        #so we know when to create new horizontal button boxes
+        b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
+        #b.set_title('Dismiss')
+        disImage = gtk.Image()
+        disImage.set_from_file("/opt/witter/icons/dismiss.png")
+        disImage.show()
+        b.set_image(disImage)
+        b.connect("clicked", self.dismissActionButtons)
+        self.builder.get_object("hbuttonbox-act4").pack_end(b, expand=True)
+
+
+        #get the selection
+        treeselection = self.treeview.get_selection()
+        select1, select2 = treeselection.get_selected_rows()
+        #entry1, entry2 = self.treeview.get_selection().get_selected()
+        #we might one day have more than on element selected, for now we get 1 row
+        try:
+            #we want to scroll to the right place in pannable area, *after* we've shown conrolls
+            #so the x/y need to be in the right scope
+            selectedItem = None
+            if select2 != None:
+
+                for item in select2[0]:
+                    #bring the selected item to the top of the view
+
+                    #we want to access field 2 (0,1,2) which has our Tweet in it
+                    entry = select1.get_value(select1.get_iter(item), 2)
+                    #and we might as well list the person who provided the url
+                    name = select1.get_value(select1.get_iter(item), 0)
+                    id = select1.get_value(select1.get_iter(item), 4)
+                    selectedItem = item
+                    #exclude logged in user from the reply_all string
+                    if (re.search(self.controller.activeAccount.getUsername().lower(), name.lower())):
+                        self.reply_all = ""
+                    else:
+                        self.reply_all = name
+                    if re.search("http", entry):
+                        #convert the string to chunks deliniated on space (we assume the url 
+                        #has spaces around it 
+                        L = string.split(entry)
+                        for word in L :
+                            #find the 'word' which is our url
+                            if re.search("http", word):
+                                url = word
+
+                                b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                  hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                                b.set_title('Open Url')
+                                b.set_value(url)
+                                b.connect("clicked", self.controller.openBrowser, url)
+                                self.builder.get_object("hbuttonbox-act3").pack_start(b, expand=True)
+                    if re.search("@", entry):
+                        L = string.split(entry)
+                        for word in L :
+                            if re.search("@", word):
+                                user = word
+                                if (user != name):
+                                    if (re.search(self.controller.activeAccount.getUsername().lower(), user.lower())):
+                                        print "excluding self from reply all"
+                                    else:
+                                        self.controller.reply_all = self.controller.reply_all + " " + user
+                                    b = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                                    b.set_title('Follow')
+                                    b.set_value(user)
+                                    b.connect("clicked", self.controller.activeAccount.FollowUser, user)
+                                    self.builder.get_object("hbuttonbox-act4").pack_start(b, expand=True)
+                                    b2 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+
+                                    b2.set_value(user)
+                                    b2.connect("clicked", self.controller.tweetAtUser, user)
+                                    self.builder.get_object("hbuttonbox-act4").pack_start(b2, expand=True)
+                                    b3 = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+
+                                    b3.set_title('History')
+                                    b3.set_value(user)
+                                    b3.connect("clicked", self.controller.showHist, user)
+                                    self.builder.get_object("hbuttonbox-act4").pack_start(b3, expand=True)
+
+
+                    #rect = self.treeview.get_background_area(item, self.tvctweet)
+                    #x, y = self.treeview.convert_bin_window_to_tree_coords(rect.x, rect.y)
+                #only provide a follow button if we aren't in our own timeline (in which case we're already following
+                curView = self.getCurrentView()
+                if (curView != self.TIMELINE_VIEW):
+                    followButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                          hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                    followButton.set_title('Follow')
+                    followButton.set_value(name)
+                    followButton.connect("clicked", self.controller.activeAccount.FollowUser, name)
+                    self.builder.get_object("hbuttonbox-act3").pack_start(followButton, expand=True)
+                unfollowButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                unfollowButton.set_title('unFollow')
+                unfollowButton.set_value(name)
+                unfollowButton.connect("clicked", self.controller.activeAccount.UnFollowUser, name)
+                self.builder.get_object("hbuttonbox-act3").pack_start(unfollowButton, expand=True)
+                histButton = hildon.Button(gtk.HILDON_SIZE_FINGER_HEIGHT,
+                                                      hildon.BUTTON_ARRANGEMENT_VERTICAL)
+                histButton.set_title('History')
+                histButton.set_value(name)
+                histButton.connect("clicked", self.controller.showHist, name)
+                self.builder.get_object("hbuttonbox-act3").pack_start(histButton, expand=True)
+
+                self.builder.get_object("AtUser").set_label(name)
+
+                self.controller.reply_to = id
+                self.controller.reply_to_name = name
+
+                self.controller.retweettext = entry
+                self.controller.retweetname = name
+                self.controller.retweetid = id
+
+        except IndexError:
+            print "nothing selected"
+
+        self.builder.get_object("hbuttonbox-act1").show_all()
+        self.builder.get_object("hbuttonbox-act2").show_all()
+        self.builder.get_object("hbuttonbox-act3").show_all()
+        self.builder.get_object("hbuttonbox-act4").show_all()
+        self.scrollToItem(selectedItem)
+
     def scrollToItem(self, item):
         try:
             rect = self.treeview.get_background_area(item, self.tvctweet)
@@ -1173,7 +1339,11 @@ class WitterUI():
          self.builder.get_object("hbox1").show_all()
          tweet = self.builder.get_object("TweetText").get_text()
          counter = self.builder.get_object("Counter")
-         counter.set_text((str(140 - len(tweet))))
+         if (140 - len(tweet) < 0):
+             counter.set_text("<span foreground=\"red\">" + (str(140 - len(tweet))) + "</span>")
+         else:
+             counter.set_text("<span foreground=\"white\">" + (str(140 - len(tweet))) + "</span>")
+         counter.set_use_markup(True)
 
 
     def about(self, widget, *args):
@@ -1702,7 +1872,7 @@ class WitterUI():
                                            hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
            Creds2.set_label("Configure Oauth")
            # Attach callback to clicked signal
-           Creds2.connect("clicked", self.controller.configOauth)
+           Creds2.connect("clicked", self.controller.configOauth, account)
            Creds2.show()
            hbox.pack_start(Creds2, False, False, 0)
            vbox.pack_start(hbox, True, True, 0)
@@ -1812,3 +1982,11 @@ class WitterUI():
 
     def setWindowTitle(self, title):
         self.window.set_title(title)
+
+    def reload_account_window(self, widget, account):
+        stack = hildon.WindowStack.get_default()
+        stack.pop(1)
+        self.edit_account_details(widget, account)
+
+
+
