@@ -71,7 +71,7 @@ class Witter():
     #first an init method to set everything up    
     def __init__(self):
         #version of witter
-        self.version = "0.3.0"
+        self.version = "0.3.1"
         #object holding witter config
         self.config = None
         #defaults for auto-refresh
@@ -82,6 +82,7 @@ class Witter():
         self.DMsRefreshInterval = 30
         self.publicRefreshInterval = 0
         self.searchRefreshInterval = 0
+        self.font_size = 18
         #we use the busy counter to track the number of busy threads
         #and show a progres/busy indicator whilst it's more than 0
 
@@ -144,10 +145,10 @@ class Witter():
         self.accounts = []
         for acc in self.config.accountList:
             if (acc.status == acc.ACTIVE):
-                self.activeAccount = account.account(self.osso_c, acc)
+                self.activeAccount = account.account(self.osso_c, acc, self)
                 self.accounts.append(self.activeAccount)
             else:
-                additional_acc = account.account(self.osso_c, acc)
+                additional_acc = account.account(self.osso_c, acc, self)
                 self.accounts.append(additional_acc)
 
         #call the refresh thread
@@ -156,6 +157,7 @@ class Witter():
         self.ui = ui.WitterUI(self)
         self.ui.setActiveListStore(self.activeAccount.getTimeline())
 
+
         self.gettingTweets = False
         if (self.activeAccount.getUsername() != "Username"):
             self.start_refresh_threads()
@@ -386,7 +388,7 @@ class Witter():
                 print "end of file, probably no token"
             try:
                 self.config = witter.Config()
-                self.config.font_size = 18
+                self.config.font_size = self.font_size
                 self.config.textcolour = "#FFFFFF"
                 self.bg_top_color = "#6bd3ff"
                 self.config.bg_bottom_color = "#0075b5"
@@ -456,7 +458,7 @@ class Witter():
             f.write("textcolour = " + self.ui.textcolour + "\n")
             f.write("bg_top = " + self.ui.bg_top_color.to_string() + "\n")
             f.write("bg_bottom = " + self.ui.bg_bottom_color.to_string() + "\n")
-            f.write("font_size = " + str(self.ui.font_size) + "\n")
+            f.write("font_size = " + str(self.font_size) + "\n")
             f.write("[refresh_interval]\n")
             f.write("timeline = " + str(self.timelineRefreshInterval) + "\n")
             f.write("mentions = " + str(self.mentionsRefreshInterval) + "\n")
@@ -515,7 +517,7 @@ class Witter():
             config.bitlyusername = self.bitlyusername
             config.bitlyapikey = self.bitlyapikey
             accData = account.accountdata()
-            activeAccount = account.account(self.osso_c, accData)
+            activeAccount = account.account(self.osso_c, accData, self)
             activeAccount.setServiceCreds(self.username, password=self.password, access_token=self.access_token)
             activeAccount.setBaseUrl(self.serviceUrlRoot)
             activeAccount.setSearchUrl(self.searchServiceUrlRoot)
@@ -526,13 +528,14 @@ class Witter():
             return config
 
 
-    def configOauth(self, *args):
+    def configOauth(self, widget, account, *args):
          try:
              twitter = oauthtwitter.OAuthApi(self.CONSUMER_KEY, self.CONSUMER_SECRET)
              self.request_token = twitter.getRequestToken()
              authorization_url = twitter.getAuthorizationURL(self.request_token)
              print authorization_url
              self.osso_rpc.rpc_run_with_defaults("osso_browser", "open_new_window", (authorization_url,))
+             self.auth_account = account
              #ask the ui to prompt user to go to browser etc.
              self.ui.showOauthPrompts()
 
@@ -562,7 +565,11 @@ class Witter():
          access_token = twitter.getAccessToken(pin)
          print access_token
          self.access_token = access_token
+         api = oauthtwitter.OAuthApi(self.CONSUMER_KEY, self.CONSUMER_SECRET, access_token)
+         self.auth_account.access_token = access_token
+         #self.activeAccount.setAccessToken(access_token)
          self.ui.builder.get_object("OauthDialog").hide_all()
+         self.ui.reload_account_window(widget, self.auth_account)
 
 
     def  store_creds(self, widget, *args):
@@ -592,7 +599,7 @@ class Witter():
     def twitPic(self, widget, *args):
         print "twitPic"
         #dialog = self.wTree.get_widget("filechooserdialog1")
-        dialog = self.builder.get_object("filechooserdialog1")
+        dialog = self.ui.builder.get_object("filechooserdialog1")
         file = dialog.get_filename()
 
         try:
@@ -692,6 +699,9 @@ class Witter():
 
     def getSearchWrapper(self, get_older=False, autoval=None, more=0, *args):
         self.ui.showBusy(1)
+        if (autoval == 1):
+            #if we manually his search get the latest content of the search box
+            searchTerms = self.ui.getEntryText()
         self.activeAccount.getSearch(auto=autoval, searchTerms=self.search_terms, older=get_older, get_count=more)
         self.ui.showBusy(-1)
 
@@ -870,7 +880,7 @@ class Witter():
 
 
     def addNewAccount(self, accountData):
-        newAccount = account.account(self.osso_c, accountData)
+        newAccount = account.account(self.osso_c, accountData, self)
         self.accounts.append(newAccount)
 
     def deleteAccount(self, accountData):
